/*
  Anschlüsse:
  *H-Brücke Strom von links nach rechts: GROUND[Motor 2] Motor 2 (BLACK), V[Motor 2] (RED), V[H-Brücke] (RED), GROUND[H-Brücke] (BLACK), V[Motor 1] (RED), GROUND[Motor 1] (BALCK)
  *H-Brücke seriell: GROUND[H-Brücke] an GROUND[Teensy] (BLACK), SO[H-Brücke] an Pin9/RX[Teensy] (GREEN), SI[H-Brücke] an Pin10/TX[Teensy] (ORANGE)
  *H-Brücke Jumper 5V (Mitte): 5V[Sensor Motor 2] (BLUE), 5V[Sensor Motor 1] (BLUE)
  *H-Brücke Jumper GROUND (links): GROUND[Sensor Motor 2] (GREEN), GROUND[Sensor Motor 1] (GREEN)
  *Teensy Pin2: Encoder A[MOTOR 2] (WHITE)
  *Teensy Pin3: Encoder B[MOTOR 2] (YELLOW)
  *Teensy Pin4: Encoder A[MOTOR 1] (WHITE)
  *Teensy Pin5: Encoder B[MOTOR 1] (YELLOW)
  *Teensy Pin6: Sirene (BLUE)
  *Teensy Pin9: SO[H-Brücke] (ORANGE)
  *Teensy Pin10: SI[H-Brücke] (GREEN)
  *Teensy Pin14: STEP[Steppersteuerung] (YELLOW)
  *Teensy Pin15: DIR[Steppersteuerung] (GREEN)
  *Teensy Pin18: SDA[IR-Camera] (RED)
  *Teensy Pin19: SCL[IR-Camera]  (ORANGE)
  *Teensy Pin21: Pumpe (PURPLE)
  
  http://www.pololu.com/file/download/TReX_Commands_v1.2.pdf?file_id=0J1
*/


#include <Arduino.h>
#include <i2c_t3.h>
#include "teensy3_Melexis90620.h"

Melexis90620 sensor;      //Initialisieren einer Infrarotkamera-Verbindung

  
//------------------------------------------------------------------------------

//globale Variablen

int motorLinksRichtung;    //0 für rückwärts und 1 für vorwärts
int motorRechtsRichtung;   //0 für rückwärts und 1 für vorwärts
int motorGeschwindigkeit;  //Werte von 0 bis 127

double tempMax;            //Die maximale Temperatur
int tempMaxAtX;            //Der X-Wert der maximalen Temperatur

unsigned char angekommen;  //0 für nicht angekommen und 1 für angekommen

//------------------------------------------------------------------------------

void setup ()
{   
  // Start der seriellen Schnittstellen
  Serial2.begin(19200);     //Schnittstelle zu den Motoren
  Serial.begin(57600);      //Schnittstelle zum Surface
  
  //Initialisierung der Motorsensorpins
  for(int i=2; i<6; i++)    //4 Pins
  {
    pinMode(i, INPUT);      //Pins für den Output der Motorencoder
  }
  
  //Initialisierung der Löschvorrichtung
  pinMode (6,OUTPUT);       //Steuerpin für die Sirene
  pinMode (14,OUTPUT);      //Steuerpin für die Steps des Steppers
  pinMode (15,OUTPUT);      //Steuerpin für die Richtung des Steppers
  pinMode (21,OUTPUT);      //Steuerpin für die Stärke der Pumpe
  
  //Initialisierung der Motoren
  Serial2.write(0x83);      //Start der Motorverbindung
  delay(100);               //Pause von 100ms
  
  //Starten der Thermalkamera
  //Serial.println ("Thermalkamera wird gestartet.");
  sensor.init (16);         //Initialisierung der Infrarotkamera an Pin 16
  //Serial.println ("Thermalkamera ist gestartet.");
  
  //Setzen der Motorvariablen auf ihren Startwert
  motorLinksRichtung = 0;   //Initialisierung der Richtung des linken Motors mit Null
  motorRechtsRichtung = 0;  //Initialisierung der Richtung des rechten Motors mit Null
  motorGeschwindigkeit = 0; //Initialisierung der Motorgeschwindigkeit mit Null
  angekommen = 0;           //Initialisierung des Wahrheitswertes angekommen mit Null
}


//------------------------------------------------------------------------------

void loop() 
{
  //Programmablauf
  input ();                   //Empfange die Befehle vom Surface
  if (angekommen == 0)        //Wenn der Roboter noch nicht an der Kerze angekommen ist...
  {
    fahren();                 //...fahre zur Kerze
  }
  else if(angekommen == 1)    //Wenn der Roboter an der Kerze angekommen ist...
  {
    loeschen();               //...lösche die Kerze
  }
  else
  {
    anhalten();              //Roboter anhalten
    digitalWrite (6, LOW);   //Sirene ausschalten
    analogWrite (21, 0);     //Pumpe ausschalten
  }
  
  //Ausgabe von Werten zur Fehleridentifizierung
  /*
  Serial.print ("Motor links Richtung: ");
  Serial.print (motorLinksRichtung);
    Serial.print ("\t");
  Serial.print ("Motor rechts Richtung: ");
  Serial.print (motorRechtsRichtung);
    Serial.print ("\t");
  Serial.print ("Motor Geschwindigkeit: ");
  Serial.print (motorGeschwindigkeit);
    Serial.print ("\t");
  Serial.println ("Höchste Temperatur: ");
  Serial.print (tempMax);
  */
}


//------------------------------------------------------------------------------

void input ()
{
  if (Serial.available() > 2)                   //Wenn neue Daten vorhanden sind...
  {
    char inByte = Serial.read();                //...speicher den Input in inByte
    switch (inByte)                             //...erkenne den Buchstaben
    {
      case 'a':                                 //Wenn ein a Empfangen wird...
      {
        unsigned char inValue;                  //...lege eine Variable für den Wert an
        Serial.readBytes((char*) &inValue, 1);  //...lese den Inhalt nach inValue ein
        motorLinksRichtung = (int) inValue;     //...schreibe den Inhalt als int nach motorLinksRichtung
        break;                                  //...breche ab
      }
      case 'b':                                 //Wenn ein b Empfangen wird...
      {
        unsigned char inValue;                  //...lege eine Variable für den Wert an
        Serial.readBytes((char*) &inValue, 1);  //...lese den Inhalt nach inValue ein
        motorRechtsRichtung = (int) inValue;    //...schreibe den Inhalt als int nach motorLinksRichtung
        break;                                  //...breche ab
      }
      case 'c':                                 //Wenn ein c Empfangen wird...
      {
        unsigned char inValue;                  //...lege eine Variable für den Wert an
        Serial.readBytes((char*) &inValue, 1);  //...lese den Inhalt nach inValue ein
        motorGeschwindigkeit = (int) inValue;   //...schreibe den Inhalt als int nach motorGeschwindigkeit
        break;                                  //...breche ab
      }
      break;                                    //Breche ab
    }
  }
}


//------------------------------------------------------------------------------

void fahren ()
{
  if (motorLinksRichtung > 0) Serial2.write(0xC2);                                       //Wenn der linke Motor vorwärts steuern soll, schicke vorwärts an den Motor
  else Serial2.write(0xC1);                                                              //Sonst schicke rückwaärts an den Motor
  Serial2.write(motorGeschwindigkeit);                                                   //Übergebe dem Motor die Geschwindigkeit
  if (motorRechtsRichtung > 0) Serial2.write(0xCA);                                      //Wenn der rechte Motor vorwärts steuern soll, schicke vorwärts an den Motor
  else Serial2.write(0xC9);                                                              //Sonst schicke rückwaärts an den Motor
  Serial2.write(motorGeschwindigkeit);                                                   //Übergebe dem Motor die Geschwindigkeit
  
  if (motorLinksRichtung == 1 && motorRechtsRichtung == 1 && motorGeschwindigkeit < 25)  //Wenn der Roboter vor der Kerze steht...
  {
    angekommen = 1;                                                                      //...setze angekommen = 1
  }
}


//------------------------------------------------------------------------------


void loeschen()
{
  //Anhalten
  anhalten();                                                //Die Geschwindigkeit der Motoren auf Null setzen
  
  //Löschen
  ircam();                                                   //Messung
  if(tempMax > 50){                                          //Wenn eine Kerze erkannt wird...
    digitalWrite(6,HIGH);                                    //...Sirene einschalten
    if (tempMaxAtX > 8) digitalWrite(15,HIGH);               //Wenn die Kerze zu weit rechts ist, Richtung des Steppers auf XY setzen
    else  digitalWrite(15,LOW);                              //Sonst die Richtung des Steppers auf links setzen
    if (tempMaxAtX == 8 || tempMaxAtX == 7) pumpen();        //Wenn die Kerze mittig steht, pumpen                          
    else drehen(1);                                          //Sonst drehen
  }
  
  //Gelöscht?
  ircam();                                                   //Messung
  if(tempMax < 50)                                           //Wenn die Temperatur unter 50 ist...
  {
    angekommen = 0;                                          //...angekommen wieder auf Null setzen
    digitalWrite (6, LOW);                                   //...Sirene ausschalten
    analogWrite (21, 0);                                     //...Pumpe ausschalten
  } 
  delay(100);                                                //Pause von 100ms
}

//------------------------------------------------------------------------------
//Suchen nach waermste Punkt

void ircam()
{
  tempMax = 0;                                         //Initialisierung der maximalen Temperatur mit Null
  tempMaxAtX = 0;                                      //Initialisierung der Position der maximalen Temperatur mit Null
  
  sensor.readTemperatures();                           //Auslesen der Temperaturdaten
  double* pixelTemps = sensor.getPixelTemperatures();  //Definieren eines Pointers auf ein Double Pixel-Array mit Temperaturdaten
  for (int x = 0; x < 16; x++){                        //Für jede Spalte...
    for (int y = 0; y < 4; y++){                       //...und jede Zeile
      if (pixelTemps[y+x*4] > tempMax){                //Wenn die Temperatur des aktuellen Pixels höher als die maximale Temperatur ist...
        tempMax = pixelTemps[y+x*4];                   //...überschreibe die maximale Temperatur
        tempMaxAtX = x;                                //...überschreibe die Position der maximalen Temperatur
      }
    }
  }
}

//------------------------------------------------------------------------------

int pumpen()
{
  digitalWrite (15,HIGH);                //Stepper auf Rechts schalten
  drehen (2);                            //2 Steps drehen
  
  for (int k=0; k<4; k++)                //Für 4 Steps
  {
    
    //Aktivieren der Pumpe und Variieren der Pumpstärke(ca. 20cm bis 30cm)
    for (int i = 80; i < 255; i += 5)    //von minimalem bis maximalem Pumpeninput
    {
      analogWrite (21, i);               //Übergabe der Laufvariable i als relativen Input für die Pumpe
      delay (10);                        //Pause von 10ms
    }
     
    // Wenn Kerze aus ist, pumpe ausschalten und funktion beenden
    ircam();                             //Messung der aktuellen Temperaturen
    if (tempMax < 50)                    //Wenn die Kerze aus ist...
    {
      analogWrite (21, 0);               //...schalte die Pumpe aus
      return 0;                          //...gehe aus der Funktion raus
    }
    
    digitalWrite (15, LOW);              //Stepper auf Rechts schalten
    drehen (1);                          //1 Steps drehen
  }
  return 0;                              //Gehe aus der Funktion raus
}


//------------------------------------------------------------------------------

void drehen(int s)
{
  for(int i = 0; i < s; i++)  //Für s Steps
  { 
    digitalWrite(14,HIGH);    //Schalte Stepper an
    delayMicroseconds(500);   //Pause von 500μs
    digitalWrite(14,LOW);     //Schalte Stepper aus
    delayMicroseconds(500);   //Pause von 500μs
  }
}


//------------------------------------------------------------------------------

void anhalten()
{
  Serial2.write(0xC2);  //Motor 1...
  Serial2.write(0);     //...stoppen
  Serial2.write(0xCA);  //Motor 2...
  Serial2.write(0);     //...stoppen
}
