#include "ofApp.h"															//Einfgen von eigenen und vorgegebenen Definitionen

//--------------------------------------------------------------
void ofApp::setup()
{
	//Auflsung der Kamera
	camWidth  = 640;														//Breite in Pixeln
	camHeight = 480;														//Hhe in Pixeln
	
    //Liste aller angeschlossenen Kameras
	vector<ofVideoDevice> devices = vidGrabber.listDevices();				//Auflistung aller angeschlossenen Kameras
    for(int i = 0; i < devices.size(); i++)									//Durchgehen aller Kameras
	{
		cout << devices[i].id << ": " << devices[i].deviceName;				//Ausgabe der Kamera-ID und des Kameranamens
        if( devices[i].bAvailable )											//Wenn die Kamera erreichbar ist...
		{
            cout << endl;													//...beende die Zeile
        }
		else																//Wenn nicht...
		{
            cout << " - unavailable " << endl;								//...fge ein "unavailable" fr "nicht erreichbar" an
        }
	}
    
	//Initialisieren der PS3-Kamera
	vidGrabber.setDeviceID(2);												//Auswhlen der PS3-Kamera (3. Kamera beim Surface, also ID = 2)
	vidGrabber.setDesiredFrameRate(60);										//Framerate auf 60 FPS setzen
	vidGrabber.initGrabber(camWidth,camHeight);								//Festlegen der Auflsung
	ofSetVerticalSync(true);												//Aktivieren von VSync

	surfaceKamera.setDeviceID(0);											//Auswhlen der Surface Frontkamera (1. Kamera beim Surface, also ID = 0)
	surfaceKamera.setDesiredFrameRate(60);									//Framerate auf 60 FPS setzen
	surfaceKamera.initGrabber(camWidth,camHeight);							//Festlegen der Auflsung
	ofSetVerticalSync(true);												//Aktivieren von VSync

	//Initialisierung der Schnittstelle
	mySerial.listDevices();													//Auflistung der seriellen Schnittstellen
	vector <ofSerialDeviceInfo> deviceList = mySerial.getDeviceList();		//Auflistung der seriellen Schnittstellen
	mySerial.setup(0, 57600);												//ffnen der Schnittstelle zum Teensy (ID = 0, Baudrate = 57600)
}


//--------------------------------------------------------------
void ofApp::update(){

}

//--------------------------------------------------------------

ofApp::Punkt sucheHellstenPunkt (unsigned char* pixels, int camHeight, int camWidth)						//bergabe des Pointers auf den ersten Pixel und die Kameraauflsung
{
	//Beginn der Messung
	int startzeile = 0;																						//Festlegen der Startzeile der Messung
	int position = (camWidth * 3) * startzeile;																//Variable fr die aktuelle Position, initialisiert mit dem Startpixel

	//Initialisierung der Punkte
	ofApp::Punkt aktuellerPunkt;																			//Anlegen eines aktuellen Punktes vom struct Punkt
	ofApp::Punkt hellsterPunkt;																				//Anlegen eines hellsten Punktes vom struct Punkt

	hellsterPunkt.x = 0;																					//Initialisieren des X-Wertes mit einer Null
	hellsterPunkt.y = 0;																					//Initialisieren des Y-Wertes mit einer Null
	hellsterPunkt.helligkeit = 0.0;																			//Initialisieren des Helligkeitswertes mit einer Null

	//Suche nach dem hellsten Punkt im Pixelarray
	for (aktuellerPunkt.y = startzeile ; aktuellerPunkt.y < camHeight; aktuellerPunkt.y++)					//Durchgehen aller Pixel in Y-Richtung
	{
		for (aktuellerPunkt.x = 0 ; aktuellerPunkt.x < camWidth; aktuellerPunkt.x++)						//Durchgehen aller Pixel in X-Richtung
		{
			aktuellerPunkt.helligkeit = pixels[position] + pixels[position + 1] + pixels[position + 2];		//Helligkeit ist die Summe des R-, G- und B-Wertes des Pixels

			if (aktuellerPunkt.helligkeit > hellsterPunkt.helligkeit)										//Wenn der aktuelle Punkt heller ist, als der aktuell hellste...
			{
				hellsterPunkt.x = aktuellerPunkt.x;															//...berschreibe den X-Wert
				hellsterPunkt.y = aktuellerPunkt.y;															//...berschreibe den Y-Wert
				hellsterPunkt.helligkeit = aktuellerPunkt.helligkeit;										//...berschreibe den Helligkeitswert
			}

			position += 3;																					//Erhhung der Position um 3, da jeder Pixel mit 3 Farbwerten gespeichert wird
		}
	}

	//Rckgabe
	return hellsterPunkt;																					//Rckgabe des hellsten Punktes
}

//-------------------------------------------------------------- 
void ofApp::draw()
{
	//Anlegen des hellsten Punktes
	ofApp::Punkt hellsterPunkt;															//Anlegen eines hellsten Punktes vom struct Punkt
	ofApp::Punkt motor;																	//Anlegen eines motor Punktes vom struct Punkt

	hellsterPunkt.x = 0;																//Initialisieren des X-Wertes mit einer Null
	hellsterPunkt.y = 0;																//Initialisieren des Y-Wertes mit einer Null
	hellsterPunkt.helligkeit = 0.0;														//Initialisieren des Helligkeitswertes mit einer Null

	//schwarzer Hintergrund
	ofBackground(0, 0, 0);																//Initialisierung des Hintergrunds mit den RGB-Werten Null
	
	//neues Bild
	vidGrabber.update();																//Aktualisierung des Bildes
	surfaceKamera.update();																//Aktualisierung des Bildes

	//Bestimmung des hellsten Punktes
	unsigned char* pixels = vidGrabber.getPixels();										//Pointer auf ein Pixelarray des Bildes
	hellsterPunkt = sucheHellstenPunkt (pixels, camHeight, camWidth);					//bergabe des Pixelarrays an die Funktion und Ausgabe des hellsten Punktes

	//Bild ausgeben
	glColor4f(1, 1, 1, 1);																//Setzen der Farbe auf Wei
	vidGrabber.draw(320, 300);															//Ausgabe des Bildes mittig auf dem Bildschirm (X+320, Y+300)
	surfaceKamera.draw(960, 300);														//Ausgabe des Bildes mittig auf dem Bildschirm (X+960, Y+300)

	//Markieren des hellsten Punktes
	glColor4f(1, 0, 0, 0.5);															//Setzen der Farbe auf Rot
	ofCircle(hellsterPunkt.x + 320, hellsterPunkt.y + 300, 10);							//Zeichnen eines roten Kreisen mit dem Radius 10 um den hellsten Punkt

	//Umrechnung auf fr die Motorsteuerung
	if ((hellsterPunkt.helligkeit / 3) > 100)											//Wenn die Helligkeit (0-255) ber 100 liegt...
	{
		motor.x = (int)((double)hellsterPunkt.x / 640.0 * 254.0 - 127);					//Umrechnung der X-Werte auf Werte zwischen -127 und 127
		motor.y = (int)((double)hellsterPunkt.y / 480.0 * 127.0);						//Umrechnung der Y-Werte auf Werte zwischen 0 und 127
	}
	else																				//Ansonsten...
	{
		motor.x = 127;																	//Reagiere so, als wre die Kerze ganz rechts im Bild...
		motor.y = 0;																	//...und ganz oben
	}

	//Motorinformationen fr den Teensy
	unsigned char motorLinksRichtung   = 0;												//Initialisierung der Richtung des linken Motors mit Null
	unsigned char motorRechtsRichtung  = 0;												//Initialisierung der Richtung des rechten Motors mit Null
	unsigned char motorGeschwindigkeit = 0;												//Initialisierung der Motorgeschwindigkeit mit Null

	//Berechnung der Fahrbefehle
	if (motor.x < -25)																	//Wenn die Kerze links von der Mitte ist (mit 25 Toleranz)...
	{
		motorLinksRichtung = 0;															//Motor links: rckwrts
		motorRechtsRichtung = 1;														//Motor rechts: vorwrts
		motorGeschwindigkeit = (unsigned char)((double)(abs (motor.x)) / 3.0 + 20.0);	//Motor Geschwindigkeit: abhngig von dem X-Wert des hellsten Punktes
	}
	else if (motor.x > 25)																//Wenn die Kerze rechts von der Mitte ist (mit 25 Toleranz)...
	{
		motorLinksRichtung = 1;															//Motor links: vorwrts
		motorRechtsRichtung = 0;														//Motor rechts: rckwrts
		motorGeschwindigkeit = (unsigned char)((double)(abs (motor.x)) / 3.0 + 20.0);	//Motor Geschwindigkeit: abhngig von dem X-Wert des hellsten Punktes
	}
	else																				//Wenn die Kerze in der Mitte ist (mit 25 Toleranz)...
	{
		motorLinksRichtung = 1;															//Motor links: vorwrts
		motorRechtsRichtung = 1;														//Motor rechts: vorwrts
		motorGeschwindigkeit = (unsigned char)((127.0 - (double)motor.y) / 2.0 + 10.0);	//Motor Geschwindigkeit: abhngig von dem Y-Wert des hellsten Punktes
	}
	
	//bertragung der Fahrbefehle an den Teensy
	mySerial.writeByte('a');															//Senden des Buchstaben 'a'
	mySerial.writeByte(motorLinksRichtung);												//Senden der Richtung des linken Motors
	mySerial.writeByte('b');															//Senden des Buchstaben 'b'
	mySerial.writeByte(motorRechtsRichtung);											//Senden der Richtung des rechten Motors
	mySerial.writeByte('c');															//Senden des Buchstaben 'c'
	mySerial.writeByte(motorGeschwindigkeit);											//Senden der Motorgeschwindigkeit

	//Auslesen der Daten aus dem Teensy
	/*
	unsigned char buffer[80]= {0};
	mySerial.readBytes(buffer,80-1);
	cout<<buffer;
	*/

	//Ausgabe der Geschwindigkeit
	/*
	cout<<(int) motorGeschwindigkeit;
	cout<<"\n";
	*/
}



//Weitere Funktionen, die fr unsere Anwendung nicht notwendig, aber fr OpenFrameworks vorhanden sein mssen

//--------------------------------------------------------------
void ofApp::keyPressed  (int key){ 

}

//--------------------------------------------------------------
void ofApp::keyReleased(int key){ 
	
}

//--------------------------------------------------------------
void ofApp::mouseMoved(int x, int y ){
	
}

//--------------------------------------------------------------
void ofApp::mouseDragged(int x, int y, int button){
	
}

//--------------------------------------------------------------
void ofApp::mousePressed(int x, int y, int button){
	
}

//--------------------------------------------------------------
void ofApp::mouseReleased(int x, int y, int button){

}

//--------------------------------------------------------------
void ofApp::windowResized(int w, int h){

}

//--------------------------------------------------------------
void ofApp::gotMessage(ofMessage msg){

}

//--------------------------------------------------------------
void ofApp::dragEvent(ofDragInfo dragInfo){ 

}